{$F+,X+}
unit zFormCaption;

{*******************************************************}
{                                                       }
{         Copyright (c) FINANCES Ltd                    }
{         2003   ver. 1                                 }
{                                                       }
{         www.finances-ltd.narod.ru                     }
{         finances-ltd@narod.ru                         }
{                                                       }
{*******************************************************}

{
ZFormCaption  for D6, D7
by FINANCES LTD (2002)


ZFormCaption is easy and useful component, created for
manipulating Form's caption.

After you have dropped  ZFormCaption component on the form,
you can use 3 - part Form's caption, each one is TLabel :

AppNameText : TLabel
CaptionText : TLabel
HintText : TLabel;

YOu can modify properties from Object Inspector at DesignTime,
or writing code , as it is shown in Demo Program.

Each TextLabel  has following properties:

Caption : string,
Font : TFont,
Visible : boolean,

that can be edited both at DesignTime and RunTime.

DELPHI VERSIONS

Tested in Delphi 6 and Delphi 7


INSTALLATION

To install component, just add zFormCaptionReg.pas file to
existing or new package, and compile it .  Enjoy .




Component is free.
You can distribute, modify source, but keeping this text is necassary !!!
Author doesn't keep any warranty of using this component.

Finances LTD
2002
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics,
  Forms, Controls, {$ifndef win32} Call32NT, {$endif} Variants, Dialogs;

const
  FWordSpacing = 3;
type

  EzFormCapError = class(Exception);
  TzFormCaption = class;

  TCustomAppNameText = class(TPersistent)
  private
    { Private declarations }
    FCaption  : String;
    FFont     : TFont;
    FOwner    : TzFormCaption;
    FVisible  : Boolean;
    FLeft     : Integer;
  protected
    { Protected declarations }
    procedure SetCaption(Value: String); virtual;
    procedure SetFont(Value: TFont);
    procedure SetVisible(Value: Boolean);
  public
    { Public declarations }
    constructor Create(AOwner: TzFormCaption);  virtual;
    destructor  Destroy; override;
    property Caption : String read FCaption write SetCaption;
    property Left: Integer read FLeft write FLeft;
  published
    { Published declarations }
    property Font : TFont read FFont write SetFont;
    property Visible : Boolean read FVisible write SetVisible;
  end;  { TAppNameText }

  TAppNameText = class(TCustomAppNameText)
  published
    property Caption : String read FCaption write SetCaption;
  end;  { same as TAppNameText, just show differently in object inspector }

  TCaptionText = class(TCustomAppNameText)
  protected
    function  GetCaption: String; virtual;
    property Caption : String read GetCaption write SetCaption;
  published
    { Published declarations }
  end;

  THintText = class(TCustomAppNameText)
  published
    property Caption : String read FCaption write SetCaption;
  end;  { same as TAppNameText, just show differently in object inspector }

  TzFormCaption = class(TComponent)
  private
    { Private declarations }
    FAppNameText  : TAppNameText;
    FCaptionText  : TCaptionText;
    FHintText     : THintText;
    FEnabled      : Boolean;
    FHooked       : Boolean;
    MyOwner       : TForm;
    MyOwnerHandle : THandle;
    FWindowActive : Boolean;
    FActiveDefined: Boolean;
    FRecreating   : Boolean;
    FUseApplTitle: Boolean;
    FOldFontName: string;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    function    GetVisibleButtons: TBorderIcons;
    procedure   ExcludeBtnRgn (var R: TRect);
    function    GetTextRect: TRect;
    function    GetTitleBarRect: TRect;
//    procedure   NewCaptionText;
    {$ifdef win32}
    procedure   Perform_NCPaint(var AMsg: TMessage);
    procedure   Perform_NCActivate(var AMsg: TMessage);
    function    Handle_WMSetCursor(var Msg: TWMSetCursor): Boolean;
    {$endif}
    function    WindowIsActive: Boolean;
    //procedure WMNCPaint(var Mes : TWMNCPaint); message WM_NCPAINT;
  protected
    { Protected declarations }
    OldWndProc  : TFarProc;
    NewWndProc  : Pointer;
    procedure   Loaded; override;
    procedure   SetEnabled(Val: Boolean);
  public
    { Public declarations }
    procedure   HookWin;
    procedure   UnhookWin;
    procedure   HookWndProc(var AMsg: TMessage);
    function    HookAppWndProc(var AMsg: TMessage): Boolean;
    constructor Create(AOwner: TComponent);  override;
    destructor  Destroy; override;
    procedure   UpdateCaption;
  published
    { Published declarations }
    property CaptionText : TCaptionText read FCaptionText write FCaptionText;
    property HintText : THintText read FHintText write FHintText;
    property AppNameText : TAppNameText read FAppNameText write FAppNameText;
    property Enabled : Boolean read FEnabled write SetEnabled default True;
    property UseApplTitle: Boolean read FUseApplTitle write FUseApplTitle;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
  end;


implementation

const
  WM_WordCapRecreateNotify = WM_USER + 17804;
  { A random number for an internal message used by WordCap exclusively. }

{$ifndef win32}
const
  SPI_GETNONCLIENTMETRICS = 41;
  SM_CXSMICON = 49;
  SM_CYSMICON = 50;

type
  TOS_Bits = (os16bit, os32bit);
  TW32LogFont = record
    lfHeight: longint;
    lfWidth: longint;
    lfEscapement: longint;
    lfOrientation: longint;
    lfWeight: longint;
    lfItalic: Byte;
    lfUnderline: Byte;
    lfStrikeOut: Byte;
    lfCharSet: Byte;
    lfOutPrecision: Byte;
    lfClipPrecision: Byte;
    lfQuality: Byte;
    lfPitchAndFamily: Byte;
    lfFaceName: array[0..lf_FaceSize - 1] of Char;
  end;

  TNONCLIENTMETRICS = record
    cbSize: longint;
    iBorderWidth: longint;
    iScrollWidth: longint;
    iScrollHeight: longint;
    iCaptionWidth: longint;
    iCaptionHeight: longint;
    lfCaptionFont: TW32LogFont;
    iSmCaptionWidth: longint;
    iSmCaptionHeight: longint;
    lfSmCaptionFont: TW32LogFont;
    iMenuWidth: longint;
    iMenuHeight: longint;
    lfMenuFont: TW32LogFont;
    lfStatusFont: TW32LogFont;
    lfMessageFont: TW32LogFont;
  end;

  TOSVERSIONINFO = record
    dwOSVersionInfoSize: longint;
    dwMajorVersion: longint;
    dwMinorVersion: longint;
    dwBuildNumber: longint;
    dwPlatformId: longint;
    szCSDVersion: array[1..128] of char;
  end;

  TW32Rect = record
    left, top, right, bottom: longint;
  end;

var
  FOS_Bits : TOS_Bits;
  NewStyleControls : Boolean;
  W32SystemParametersInfo:
    function(uiAction: longint; uiParam:longint; pvParam:TNonClientMetrics; fWinIni:longint; id:longint):longint;
  W32GetSystemMetrics:
    function(index: longint; id:longint):longint;
  GetVersionEx:
    function(pvParam:TOSVersionInfo; id:longint):longint;
  CopyImage:
    function(HImage, uType, cX, cY, flags :longint; id:longint):longint;
  DrawIconEx:
    function(HDC, left, top, HIcon, Width, Height, frame, FlickFreeBrush, Flags: longint; id:longint):longint;
  DrawFrameControl:
    function(HWND, R: TW32Rect; uType, uState: longint; id:longint): longint;
  id_W32SystemParametersInfo : Longint;
  id_W32GetSystemMetrics : Longint;
  id_W32GetVersionEx : Longint;
  id_W32CopyImage : Longint;
  id_W32DrawIconEx : Longint;
  id_W32DrawFrameControl : Longint;
{$endif}

constructor TCustomAppNameText.Create(AOwner: TzFormCaption);

begin
  inherited Create;
  FOwner := AOwner;
  FFont := TFont.Create;
  //GetSystemFont(FFont);

  FFont.Assign(TForm(FOwner.Owner).Font);
  FVisible := true;
  FCaption := '';
end;

destructor TCustomAppNameText.Destroy;
begin
  FFont.Free;
  inherited destroy;
end;

procedure TCustomAppNameText.SetCaption(Value: String);
begin
  If FCaption = Value then exit;
  FCaption := Value;
//  FOwner.NewCaptionText;
  //if csDesigning in FOwner.ComponentState then
  FOwner.UpdateCaption;
end;  { TAppNameText.SetCaption }

procedure TCustomAppNameText.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
  if csDesigning in FOwner.ComponentState then FOwner.UpdateCaption;
end;  { TAppNameText.SetFont }

procedure TCustomAppNameText.SetVisible(Value: Boolean);
begin
  If FVisible = Value then exit;
  FVisible := Value;
//  FOwner.NewCaptionText;
  if csDesigning in FOwner.ComponentState then FOwner.UpdateCaption;
end;   { TAppNameText.SetVisible }

{------------------------------------------------------------------------------}
{  THintText Component                                                      }
{------------------------------------------------------------------------------}
function TCaptionText.GetCaption: String;
var temp : string;
//    found : integer;
begin
  try
    if FOwner.MyOwner = nil then begin result := ''; exit; end;
    temp := FOwner.MyOwner.Caption;
    {If FOwner.FAppNameText.Visible then
    begin
      found := Pos(FOwner.FAppNameText.Caption, Temp);
      if found <> 0 then temp := Copy(temp, found + length(FOwner.FAppNameText.Caption), maxint);
      if length(temp) > 0 then if temp[1] = ' ' then temp := Copy(temp, 2, maxint);
    end;
    If FOwner.FCaptionText.Visible then
    begin
      found := Pos(FOwner.FCaptionText.Caption, Temp);
      if found <> 0 then temp := Copy(temp, found + length(FOwner.FCaptionText.Caption), maxint);
      if length(temp) > 0 then if temp[1] = ' ' then temp := Copy(temp, 2, maxint);
    end;}
    result := temp;
  except
    {$ifdef win32} on EAccessViolation do result := '';
    {$endif}
  end;
end;

{------------------------------------------------------------------------------}
{  TzFormCaption  Component                                                 }
{------------------------------------------------------------------------------}
constructor TzFormCaption.Create(AOwner: TComponent);
  function WinDir: string;
  var s:string;
  begin
    setlength(s, max_path);
    setlength(s, GetWindowsDirectory(pchar(s), max_path));
    Result := s + '\';
  end;

begin
  inherited Create(AOwner);
  with AOwner as TForm do MyOwner := TForm(AOwner);  { My pointer to my owner form }
  MyOwnerHandle := MyOwner.Handle;
  FWindowActive := true;  { assumption }
  FActiveDefined := false;
  FUseApplTitle := True;

  FOldFontName := MyOwner.Font.Name;

  FAppNameText := TAppNameText.Create(self);
  FAppNameText.FFont.Name := 'Tahoma';
  FAppNameText.FFont.Style := [fsBold];
  FAppNameText.FFont.Color := clYellow;

  FCaptionText := TCaptionText.Create(self);
  FCaptionText.FFont.Color := clCaptionText;
  FCaptionText.FFont.Style := [fsBold];

  FHintText := THintText.Create(self);
  FHintText.FFont.Name := 'Arial';
  FHintText.FFont.Color := clCaptionText;
  FHintText.FFont.Style := [];

  FEnabled := True;
  FHooked  := false;
  FRecreating := false;
  FEnabled := True;
  Hookwin;
  if csdesigning in ComponentState then
  if not (csReadingState in MyOwner.ControlState) then
  begin
    { Set default fonts unless stored user settings are being loaded }
    FAppNameText.FCaption := '';
    FCaptionText.FCaption := MyOwner.Caption;
    FHintText.FCaption := '';
//    NewCaptionText;
    ReDrawWindow(MyOwnerHandle,nil,0,RDW_FRAME or RDW_INVALIDATE or RDW_UPDATENOW);
  end;
end;  { TzFormCaption.Create }

procedure TzFormCaption.loaded;
begin
  inherited loaded;
  { some people have reported problems with TForm's position being poScreenCenter.
    this removes the problem (I believe - I've never replicated the problem so I
    can't test it). }
  If MyOwnerHandle <> MyOwner.Handle then
  begin
    UnhookWin;
    HookWin;
  end;
end;

destructor TzFormCaption.Destroy;
begin
  UnHookWin;
  { update caption if the parent form is not being destroyed }
  If not (csDestroying in MyOwner.ComponentState) then
  begin
    MyOwner.caption := FCaptionText.Caption;
    UpdateCaption;
    MyOwner.Font.OnChange := nil;
  end;
  FCaptionText.Free;
  FHintText.Free;
  FAppNameText.Free;
  inherited destroy;  {Call default processing.}
end;  { TzFormCaption.Destroy }

procedure TzFormCaption.HookWin;
begin
  MyOwnerHandle := MyOwner.Handle;
  OldWndProc := TFarProc(GetWindowLong(MyOwnerHandle, GWL_WNDPROC));
  NewWndProc := MakeObjectInstance(HookWndProc);
  SetWindowLong(MyOwnerHandle, GWL_WNDPROC, LongInt(NewWndProc));
  If not FRecreating then Application.HookMainWindow(HookAppWndProc);
  FRecreating := false;
  FHooked := true;
end;  { HookWin }

procedure TzFormCaption.UnhookWin;
begin
  If not FHooked then exit;  { don't ever unhook a non-hooked window }
  If not FRecreating then Application.UnhookMainWindow(HookAppWndProc);
  SetWindowLong(MyOwnerHandle, GWL_WNDPROC, LongInt(OldWndProc));
  if assigned(NewWndProc) then FreeObjectInstance(NewWndProc);
  NewWndProc := nil;
  FHooked := false;
end;  { UnHookWin }

function TzFormCaption.WindowIsActive: Boolean;
begin
  If FActiveDefined then begin Result := FWindowActive; exit; end;
  Result := (MyOwnerHandle = GetActiveWindow);
  If (MyOwner.FormStyle = fsMDIChild)
    then if Application <> nil
    then if Application.Mainform <> nil
    then if MyOwner = Application.Mainform.ActiveMDIChild
    then if Application.Mainform.HandleAllocated
    then if Application.Mainform.Handle = GetActiveWindow
      then result := true;
end;  { WindowIsActive }

{$ifdef win32}
procedure TzFormCaption.Perform_NCPaint(var AMsg: TMessage);
var
  WR : TRect;
  MyRgn : HRgn;
  ACanvas : TCanvas;
  t, TotalTextWidth, w1, w2, w3: Integer;
  c1, c2, c3: string;
  OldStyle: TBrushStyle;
  FActive: Boolean;

  procedure Paint_Text(FText: TCustomAppNameText; FCapt: string);
  begin
    if FText.FCaption = '' then Exit;
    with ACanvas do begin
      Font.Assign(FText.FFont);
      if {not (csDesigning in ComponentState) and} not FActive then Font.Color := clInactiveCaptionText;


    if not (csDesigning in ComponentState) then
      TextOut(FText.Left, Round((GetSystemMetrics(SM_CYCAPTION) - Abs(Font.Height))/2) +
                                 GetSystemMetrics(SM_CYEDGE), FCapt);
    end;
  end;

Label TryAgayn;

begin
  //UpdateCaption;
  //ReDrawWindow(MyOwnerHandle,nil,0,RDW_FRAME or RDW_INVALIDATE or RDW_UPDATENOW);
  if not FEnabled then begin
    if (MyOwner.Caption = '') then
      MyOwner.Caption := FCaptionText.FCaption;
    AMsg.Result := CallWindowProc(OldWndProc,MyOwnerHandle, AMsg.Msg, MyRgn, AMsg.lParam);
    Exit;
  end;

  if MyOwner.Caption <> '' then begin
    FCaptionText.FCaption := (Owner as TForm).Caption;
    if not (csDesigning in ComponentState) then
      (Owner as TForm).Caption := '';
  end;
  GetWindowRect(MyOwnerHandle, WR);
  MyRgn := CreateRectRgnIndirect(WR);
  AMsg.Result := CallWindowProc(OldWndProc,MyOwnerHandle, AMsg.Msg, MyRgn, AMsg.lParam);

  if (FAppNameText.FCaption = '') and FUseApplTitle and
              not (csDesigning in ComponentState) then
    FAppNameText.FCaption := Application.Title
  else if (FAppNameText.FCaption = Application.Title) and not FUseApplTitle then
    FAppNameText.FCaption := '';

  wr := GetTextRect;

  if ( ((biSystemMenu in MyOwner.BorderIcons) and (MyOwner.BorderStyle in [bsSingle, bsSizeable]))
      or (csDesigning in ComponentState) ) then
    Inc(WR.Left, 22)
  else Inc(WR.Left, GetSystemMetrics(SM_CXFIXEDFRAME));

  Inc(WR.Left, 3);   // XP need

  FActive := WindowIsActive;

  if not (csDesigning in ComponentState) then
    c1 := FAppNameText.FCaption
  else c1 := '';
  c2 := FCaptionText.FCaption;
  c3 := FHintText.FCaption;

  ACanvas := TCanvas.Create;

  try
    ACanvas.Handle := GetWindowDC(MyOwnerHandle);

    //d
    if ( ((biSystemMenu in MyOwner.BorderIcons) and (MyOwner.FormStyle = FsMDIChild) and (MyOwner.BorderStyle in [bsDialog]))
      or (csDesigning in ComponentState) ) then begin
      ACanvas.CopyRect(
      Rect(GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CYFIXEDFRAME), GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYFIXEDFRAME),
      GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXFIXEDFRAME) + GetSystemMetrics(SM_CXSMICON) + 1  ,  GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYFIXEDFRAME) + GetSystemMetrics(SM_CXSMICON)+ 2),
      ACanvas,
      Rect(GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXFIXEDFRAME) + GetSystemMetrics(SM_CXSMICON) + 1 , GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYFIXEDFRAME),
      GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXFIXEDFRAME) + 2*GetSystemMetrics(SM_CXSMICON) + 1,  GetSystemMetrics(SM_CYMENUSIZE))
      );
      end;
    //d

    with ACanvas do begin
      TryAgayn:
      if FAppNameText.Visible and (c1 <> '') then begin
        Font.Assign(FAppNameText.FFont);
        w1 := TextWidth(c1 + '   ');
      end
      else w1 := 0;
      if FCaptionText.Visible and (c2 <> '') then begin
        Font.Assign(FCaptionText.FFont);
        w2 := TextWidth(c2 + '   ');
      end
      else w2 := 0;
      if FHintText.Visible and (c3 <> '') then begin
        Font.Assign(FHintText.FFont);
        w3 := TextWidth(c3);
      end
      else w3 := 0;

      TotalTextWidth := w1 + w2 + w3;

      t := WR.Right - WR.Left;

      FAppNameText.Left := WR.Left;

      if not FAppNameText.Visible then begin
        FCaptionText.Left := WR.Left;
        if FCaptionText.Visible then
          FHintText.Left := WR.Left + w2
        else FHintText.Left := WR.Left;
      end
      else begin
        FCaptionText.Left := WR.Left + w1;
        if FCaptionText.Visible then
          FHintText.Left := FCaptionText.Left + w2
        else FHintText.Left := FCaptionText.Left;
      end;

      if TotalTextWidth > t then begin
        if FHintText.Visible and (c3 <> '') then begin
          if TotalTextWidth - w3 > t then begin
            c3 := '';
            GoTo TryAgayn;
          end;
        end
        else if FCaptionText.Visible and (c2 <> '') then begin
          if TotalTextWidth - w2 > t then begin
            c2 := '';
            GoTo TryAgayn;
          end;
        end;
      end;

      OldStyle := Brush.Style;
      Brush.Style := bsClear;

      if FAppNameText.Visible and (c1 <> '') then begin
        if w1 > t then begin
          while (Length(c1) > 0) and (TextWidth(c1 + '...') > t) do
            SetLength(c1, Length(c1) - 1);
          if c1 <> '' then c1 := c1 + '...';
        end;
        if c1 <> '' then  Paint_Text(FAppNameText, c1);
      end;

      if FCaptionText.Visible and (c2 <> '') then begin
        if w1 + w2 > t then begin
          while (Length(c2) > 0) and (w1 + TextWidth(c2 + '...') > t) do
            SetLength(c2, Length(c2) - 1);
          if c2 <> '' then c2 := c2 + '...';
        end;
        if c2 <> '' then Paint_Text(FCaptionText, c2);
      end;

      if FHintText.Visible and (c3 <> '') then begin
        if w1 + w2 + w3 > t then begin
          while (Length(c3) > 0) and (w1 + w2 + TextWidth(c3 + '...') > t) do
            SetLength(c3, Length(c3) - 1);
          if c3 <> '' then c3 := c3 + '...';
        end;
        if c3 <> '' then Paint_Text(FHintText, c3);
      end;

      Brush.Style := OldStyle;
    end;
  finally
    ReleaseDC(MyOwnerHandle, ACanvas.Handle);
    ACanvas.Free;
  end;
end;  { perform_NCPaint for win32 }

procedure TzFormCaption.Perform_NCActivate(var AMsg: TMessage);
var TempMessage : TMessage;
begin
//  if not MyOwner.Visible then begin
//  CallWindowProc(OldWndProc, MyOwnerHandle, AMsg.Msg, AMsg.wParam, AMsg.lParam);
//  exit;
//  end;

  FWindowActive := TWMNCActivate(AMsg).Active;
  FActiveDefined := true;
  TempMessage.Msg := WM_NCPaint;

 case  MyOwner.FormStyle of

  fsMDIChild :

  begin
  CallWindowProc(OldWndProc, MyOwnerHandle, AMsg.Msg, AMsg.wParam, AMsg.lParam);
  Perform_NCPaint(TempMessage);
  end;

  fsMDIForm :
    begin
      if Application <> nil
      then if Application.Mainform <> nil
      then if Application.Mainform.ActiveMDIChild <> nil
      then PostMessage(Application.Mainform.ActiveMDIChild.Handle, WM_NCACTIVATE,
                       longint(TWMNCActivate(AMsg).Active), 0);
  ReDrawWindow(MyOwnerHandle,nil,0,RDW_FRAME or RDW_INVALIDATE or RDW_UPDATENOW);
  end;

  fsNormal :
    begin
    CallWindowProc(OldWndProc, MyOwnerHandle, AMsg.Msg, AMsg.wParam, AMsg.lParam);
    Perform_NCPaint(TempMessage);
    end;

  end; // case

  { Tell windows that we have handled the message }
  AMsg.Result := 1;
  AMsg.wParam := 1;

  if FWindowActive then begin
    if Assigned(FOnActivate) then FOnActivate(MyOwner)
  end
  else if Assigned(FOnDeactivate) then FOnDeactivate(MyOwner)
end;  { perform_NCActivate for win32 }
{$endif}

procedure TzFormCaption.HookWndProc(var AMsg: TMessage);
begin
  if csDestroying in MyOwner.ComponentState then Exit;


  {$ifdef win32}
  if AMsg.Msg = WM_NCPAINT then
    begin Perform_NCPaint(AMsg); exit; end; { NCPaint is handled for win32 }
    if AMsg.Msg = WM_NCACTIVATE then
    begin Perform_NCActivate(AMsg); exit; end; { NCActivate is handled for win32 }
    if AMsg.Msg = WM_SETCURSOR then
    begin if Handle_WMSetCursor(TWMSetCursor(AMsg)) then exit; end; { SetCursor is handled for win32 }
  {$endif}

  if AMsg.Msg = WM_DESTROY then begin
    {Note: WM_DESTROY is trapped here when the form itself is destroyed,
        and whenever the RecreateWnd method of the form is called }
    if not (csDestroying in ComponentState) then
    begin
      { We must unhook the WindowProc, and then rehook it later }
      FRecreating := True;
      UnHookWin;
      { Notify WordCap to rehook the form. A message must be posted so that this
        can be done once the form has completed the recreation process. }
      PostMessage (Application.Handle, WM_WordCapRecreateNotify, 0, Longint(Self));
      { don't exit.  Allow default processing to still occur }
    end
    else Exit;
   end;

  { now handle all other calls }

  AMsg.Result := CallWindowProc(OldWndProc, MyOwnerHandle, AMsg.Msg, AMsg.wParam, AMsg.lParam);

  if AMsg.Msg = WM_SETICON then
    Perform_NCPaint(AMsg);
  if ((AMsg.Msg = WM_DISPLAYCHANGE)  or
      (AMsg.Msg = WM_SysColorChange) or
      (AMsg.Msg = WM_WININICHANGE) or
      (AMsg.Msg = WM_SETTINGCHANGE) or
      (AMsg.Msg = WM_COMMAND)) then
  begin
    UpdateCaption;  {force a NC region redraw};
  end;

end;  { HookWndProc }

function TzFormCaption.HookAppWndProc(var AMsg: TMessage): Boolean;
begin
  Result := False;
  if AMsg.Msg = WM_WordCapRecreateNotify then begin
    if AMsg.LParam <> longint(self) then exit;    { did the message come from this instance or another instance? }
    HookWin;  { Rehook the form }
    If GetActiveWindow = MyOwnerHandle then FWindowActive := true;
    UpdateCaption; {}
  end;
end;

procedure TzFormCaption.UpdateCaption;
begin
//{$ifdef win32}  LockWindowUpdate(MyOwnerHandle);  {$endif}
  SetWindowPos( MyOwnerHandle, 0, 0, 0, 0, 0,
                SWP_FRAMECHANGED or SWP_DRAWFRAME or
                SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE);
                //{$ifdef win32}  LockWindowUpdate(0);  {$endif}
end;  { UpdateCaption }

(*procedure TzFormCaption.NewCaptionText;
begin
  {$ifdef win32}  LockWindowUpdate(MyOwnerHandle);  {$endif}
  {$ifdef win32}  LockWindowUpdate(0);  {$endif}
end;*)  { TzFormCaption.NewCaptionText }

function TzFormCaption.GetTitleBarRect: TRect;
var BS : TFormBorderStyle;
begin
  BS:= MyOwner.BorderStyle;
  if csDesigning in ComponentState then BS:= bsSizeable;
  { if we have no border style, then just set the rectangle empty. }
  if BS = bsNone then begin SetRectEmpty(Result); exit; end;

  GetWindowRect(MyOwnerHandle, Result);
  { Convert rect from screen (absolute) to client (0 based) coordinates. }
  OffsetRect(Result, -Result.Left, -Result.Top);
  { Shrink rectangle to allow for window border.  We let Windows paint the border. }
  { this catches drawing MDI minimised windows caption bars in Win95 }
  if IsIconic(MyOwnerHandle)
    then begin
      {$ifdef win32}
      If NewStyleControls
        then InflateRect(Result, -GetSystemMetrics(SM_CXFIXEDFRAME),
                                 -GetSystemMetrics(SM_CYFIXEDFRAME))
        else {$endif} InflateRect(Result, -GetSystemMetrics(SM_CYBORDER)-GetSystemMetrics(SM_CXDLGFRAME),
                                 -GetSystemMetrics(SM_CYBORDER)-GetSystemMetrics(SM_CYDLGFRAME));
    end
  else
  case BS of
    {$ifdef win32} bsToolWindow, bsSingle, bsDialog : //D
        InflateRect(Result, -GetSystemMetrics(SM_CXFIXEDFRAME),
                            -GetSystemMetrics(SM_CYFIXEDFRAME));
    bsSizeToolWin, bsSizeable:
        InflateRect(Result, -GetSystemMetrics(SM_CXSIZEFRAME),
                            -GetSystemMetrics(SM_CYSIZEFRAME));
    {$else}
    bsDialog:
        InflateRect(Result, -(GetSystemMetrics(SM_CXBORDER)+GetSystemMetrics(SM_CXDLGFRAME)),
                            -(GetSystemMetrics(SM_CYBORDER)+GetSystemMetrics(SM_CYDLGFRAME)) );
    bsSingle:
        InflateRect(Result, -GetSystemMetrics(SM_CXBORDER),
                            -GetSystemMetrics(SM_CYBORDER));
    bsSizeable:
        InflateRect(Result, -GetSystemMetrics(SM_CXFRAME),
                            -GetSystemMetrics(SM_CYFRAME));
    {$endif}
   end;

  { Set the appropriate height of caption bar. }
  {$ifdef win32}
  if BS in [bsToolWindow, bsSizeToolWin] then
    Result.Bottom := Result.Top + GetSystemMetrics(SM_CYSMCAPTION) - 1
  else {$endif}
    Result.Bottom := Result.Top + GetSystemMetrics(SM_CYCAPTION) - 1;
  {$ifndef win32} Result.Bottom := Result.Bottom-1; {$endif}
end;  { TzFormCaption.GetTitleBarRect }

function TzFormCaption.GetVisibleButtons: TBorderIcons;
var BS : TFormBorderStyle;
begin
  Result := [];
  if csDesigning in ComponentState then begin result := [biSystemMenu, biMaximize, biMinimize]; exit; end;
  BS:= MyOwner.BorderStyle;
  if BS = bsNone then exit;
  {$ifdef win32}
  if not (biSystemMenu in MyOwner.BorderIcons) then exit;  { none will be visible }
  if BS in [bsToolWindow, bsSizeToolWin]
  then begin
    Result := [biSystemMenu];  { close icon only }
    exit;
  end;
  {$endif}

  { ???? check this carefully for 16-bit accuracy ???? }
  if (NewStyleControls {$ifdef win32} and (biSystemMenu in MyOwner.BorderIcons) {$endif} )
    then Result := [biSystemMenu];  { close icon - this is OS dependant }
  {$ifdef win32}
  if ((BS = bsDialog) and (biHelp in MyOwner.BorderIcons) and (biSystemMenu in MyOwner.BorderIcons))
    then Result := Result + [biHelp];  { help icon }
  if ((BS = bsSingle) and (biHelp in MyOwner.BorderIcons)
      and (not(biMinimize in MyOwner.BorderIcons))
      and (not(biMaximize in MyOwner.BorderIcons)) )
    then Result := Result + [biHelp];  { help icon }
  if ((BS = bsSizeable) and (biHelp in MyOwner.BorderIcons)
      and (not(biMinimize in MyOwner.BorderIcons))
      and (not(biMaximize in MyOwner.BorderIcons)) )
    then Result := Result + [biHelp];  { help icon }
  {$endif}
  if BS = bsDialog then exit;  { no chance of Min&Max buttons }
  if NewStyleControls then
  begin
    if ((biMinimize in MyOwner.BorderIcons) or (biMaximize in MyOwner.BorderIcons))
      then Result := Result + [biMinimize, biMaximize];  { minimise and maximise button }
  end
  else begin
    if (biMinimize in MyOwner.BorderIcons)
      then Result := Result + [biMinimize];  { minimise button }
    if (biMaximize in MyOwner.BorderIcons)
      then Result := Result + [biMaximize];  { maximise button }
  end;
end;  { TzFormCaption.GetVisibleButtons }

procedure TzFormCaption.ExcludeBtnRgn (var R: TRect);
var BtnWidth: integer;
    BI : TBorderIcons;
begin
  if ((MyOwner.BorderStyle = bsNone) and (not(csDesigning in ComponentState))) then exit;
  {$ifdef win32}
  if ((MyOwner.BorderStyle in [bsToolWindow, bsSizeToolWin]) and (not(csDesigning in ComponentState)))
    then BtnWidth := GetSystemMetrics(SM_CXSMSIZE)
    else {$endif} BtnWidth := GetSystemMetrics(SM_CXSIZE);

  BI := GetVisibleButtons;
  if (biSystemMenu in BI) then R.Right := R.Right - BtnWidth - 2; { close icon }
  if (biMinimize in BI) then R.Right := R.Right - BtnWidth;  { minimize icon }
  if (biMaximize in BI) then R.Right := R.Right - BtnWidth;  { maximize icon }
  {$ifdef win32}
  if (biHelp in BI) then R.Right := R.Right - BtnWidth - 2;  { help icon }
  {$endif}

if not NewStyleControls then
if ( ( (biSystemMenu in MyOwner.BorderIcons) and (MyOwner.BorderStyle in [bsSingle, bsSizeable]) ) or
  (csDesigning in ComponentState) )
    then R.Left := R.Left + BtnWidth;  { let windows do the system icon in win3 style }

end;  { TzFormCaption.ExcludeBtnRgn }

function TzFormCaption.GetTextRect: TRect;
begin
  result := GetTitleBarRect;
  ExcludeBtnRgn(result);

  If result.Right <= result.Left then {error}
    result.Right := result.Left+2;  { right must be greater than left- otherwise system resources get lost }
end;  { GetTextRect }

procedure TzFormCaption.SetEnabled(Val: Boolean);
begin
  If Val <> FEnabled then
  begin
    FEnabled := Val;
    If not (csDesigning in ComponentState) then begin
      UpdateCaption;
    end;
  end;
end;  { SetEnabled }

{$ifdef win32}
function TzFormCaption.Handle_WMSetCursor(var Msg: TWMSetCursor): Boolean;
{returns true if we handled the message }
begin
  { Tell Windows we handled the message }
  Msg.Result := 1;
  { Load and display the correct cursor for the border area being hit }
  case Msg.HitTest of
    HTTOP,
    HTBOTTOM:      SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENS)));
    HTLEFT,
    HTRIGHT:       SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZEWE)));
    HTTOPRIGHT,
    HTBOTTOMLEFT:  SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENESW)));
    HTTOPLEFT,
    HTBOTTOMRIGHT: SetCursor(LoadCursor(0, MakeIntResource(IDC_SIZENWSE)));
  else
    { Wasn't anything we cared about, so tell Windows we didn't handle it. }
    Msg.Result := 0;
    inherited;
  end;
  result := (Msg.Result = 1);
end;
{$endif}

{$ifndef win32}
procedure Set_NewStyleControls;
var win32Ver : TOSVersionInfo;
begin
  NewStyleControls := false;  { assumption }
  Win32Ver.dwOSVersionInfoSize := sizeof(TOSVersionInfo);
  If boolean(GetVersionEx(Win32Ver, id_W32GetVersionEx)) then
    NewStyleControls := Win32Ver.dwMajorVersion >= 4;
end;  { Set_NewStyleControls }

initialization
  { set up the Win32 API function access for a 16 bit app on a 32 bit OS }
  @GetVersionEx:=@Call32;
  @W32SystemParametersInfo:=@Call32;
  @W32GetSystemMetrics:=@Call32;
  @CopyImage := @Call32;
  @DrawIconEx := @Call32;
  @DrawFrameControl := @Call32;
  id_W32GetVersionEx:=Declare32('GetVersionEx', 'kernel32', 'p');
  id_W32SystemParametersInfo:=Declare32('SystemParametersInfo', 'user32', 'iipi');
  {Check if everything went well. Call32NTError=false means no errors at all}
  if Call32NTError then begin
    NewStyleControls := false;  { a safe assumption }
    FOS_Bits := os16bit; { one or more 32 bit functions failed - so it's probably a 16bit OS }
  end else begin
    Set_NewStyleControls;
    FOS_Bits := os32bit; { all 32 bit functions worked - so it's definitely a 32bit OS }
  end;
  { Icon routines not available on Win32s - so test separately }
  id_W32GetSystemMetrics:=Declare32('GetSystemMetrics', 'user32', 'i');
  id_W32CopyImage:=Declare32('CopyImage', 'user32', 'iiiii');
  id_W32DrawIconEx:=Declare32('DrawIconEx', 'user32', 'iiiiiiiii');
  id_W32DrawFrameControl:=Declare32('DrawFrameControl', 'user32', 'wpii');
  NewStyleControls := NewStyleControls and not Call32NTError;
{$endif}
end.

